--[[
==============================================================================
Shipviewer 1.1.1 - Plugin for Vendetta Online (all platforms)
==============================================================================
Created by Snib in July 2022

This plugin provides a viewer for all ships currently in the game, incl. their
liveries and some of their stats (not all stats can be accessed by plugins).
Access the plugin using this chat command:
   /shipviewer

If your platform has trouble displaying the plugin GUI correctly, in
particular the dropdown with the ship names, you can use the command
/shipviewer2 instead to force the alternate UI. Please let me know if you are
forced to do this. Mobile platforms and small screens will always use the
alternate UI.

The ships will use the color you last selected in the Ship > Buy > Buy Ship
tab of the game. Coloring of hive ships has been disabled in this viewer,
however.

Ships released in future game updates will require an update to this plugin,
but changes to ship stats will usually not.

Ships in the non-playable categories are only included for their models, stats
could have been included but were intentionally withheld.

There's also a second command to view arbitrary other models in the game, if
you can provide their mesh name and file:
  /modelviewer <MeshName> <MeshFile>
==============================================================================
]]

declare("Shipviewer", {})
Shipviewer = { Dialog = nil }

function Shipviewer.CreateDialog(shiplist, small_screen)
    local dialog
    local shipIdx = 1
    local stats_rowcount = 0
    local liveries_rowcount = 0
    local liveries_row2overlayindex = {}
    small_screen = small_screen or (Platform == "Android" or Platform == "iOS" or
        (HUDSizeX(1) / Font.Default) <= 60)

    local shiptypes = { "playable", "npc", "hive", "hulks", "old_models", "developer", }

    local ships_dropdown = iup.stationsublist {
        dropdown = "YES",
        expand = "HORIZONTAL",
        action = function(self, text, index, state)
            if state == 1 then
                shipIdx = index
                dialog:SetShipByIndex(shipIdx)
            end
        end
    }
    local function populate_ships_dropdown()
        for i, shipdata in ipairs(shiplist) do ships_dropdown[i] = shipdata.name end
    end

    populate_ships_dropdown()

    local shiptype_dropdown = iup.stationsublist {
        dropdown = "YES",
        size = Font.Default * 8,
        action = function(self, text, index, state)
            if state == 1 then
                for i = 1, #shiplist do ships_dropdown[i] = nil end
                shiplist = Shipviewer.ships[ shiptypes[index] ]
                populate_ships_dropdown()
                shipIdx = 1
                dialog:SetShipByIndex(shipIdx)
            end
        end
    }
    for i, type in ipairs(shiptypes) do
        shiptype_dropdown[i] = type:upper():gsub("_", " ")
    end

    local liveries_dropdown = iup.stationsublist {
        dropdown = "YES",
        size = Font.Default * 8,
        action = function(self, text, index, state)
            if state == 1 then
                if index < 3 then
                    self.value = 1
                end
                dialog:SetLivery(liveries_row2overlayindex[index])
            end
        end
    }
    local stats_dropdown = iup.stationsublist {
        dropdown = "YES",
        size = Font.Default * 15,
        action = function(self) self.value = 1 end
    }
    local button_next = iup.stationbutton {
        title = "Next", fgcolor = tabunseltextcolor,
        action = function()
            shipIdx = math.fmod(shipIdx + 1, #shiplist + 1)
            if shipIdx == 0 then shipIdx = 1 end
            dialog:SetShipByIndex(shipIdx)
        end
    }
    local button_previous = iup.stationbutton {
        title = "Prev", fgcolor = tabunseltextcolor,
        action = function()
            shipIdx = math.fmod(shipIdx - 1, #shiplist + 1)
            if shipIdx == 0 then shipIdx = #shiplist end
            dialog:SetShipByIndex(shipIdx)
        end
    }
    local button_close = iup.stationbutton {
        title = "Close",
        action = function()
            dialog:Close()
        end
    }
    local modelview = iup.modelview { expand = "YES" }
    local dropdown_box_top
    local dropdown_box_bottom
    if small_screen then
        dropdown_box_top = iup.vbox {
            iup.fill { size = 20 },
            iup.hbox {
                iup.fill {},
                iup.fill { size = 20 },
                shiptype_dropdown,
                iup.fill { size = 10 },
                ships_dropdown,
                iup.fill { size = 20 },
                iup.fill {},
            }
        }
        dropdown_box_bottom = iup.hbox {
            liveries_dropdown,
            iup.fill { size = 10 },
            stats_dropdown,
        }
    else
        dropdown_box_top = iup.vbox {}
        dropdown_box_bottom = iup.hbox {
            shiptype_dropdown,
            iup.fill { size = 10 },
            ships_dropdown,
            iup.fill { size = 10 },
            liveries_dropdown,
            iup.fill { size = 10 },
            stats_dropdown,
        }
    end
    dialog = iup.dialog {
        iup.pdasubframebg {
            iup.vbox {
                dropdown_box_top,
                modelview,
                iup.vbox {
                    iup.hbox {
                        iup.fill {},
                        iup.fill { size = 20 },
                        dropdown_box_bottom,
                        iup.fill { size = 10 },
                        iup.hbox {
                            button_next,
                            button_previous,
                        },
                        button_close,
                        iup.fill { size = 20 },
                        iup.fill {},
                        alignment = 'ACENTER',
                    },
                    iup.fill { size = 15 },
                    alignment = 'ACENTER',
                },
            },
        },
        expand = "NO",
        fullscreen = "YES",
        topmost = "YES",
        defaultesc = button_close,
        show_cb = function() dialog:SetShipByIndex(shipIdx) end,
    }

    function dialog:SetShipByIndex(index, colorid)
        dialog:SetShip(
            shiplist[index].meshname,
            shiplist[index].meshfile,
            shiplist[index].typeid,
            colorid
        )
        ships_dropdown.value = shipIdx
    end

    function dialog:SetShip(meshname, meshfile, typeid, colorid)
        local shiptype = shiptypes[tonumber(shiptype_dropdown.value)]
        meshfile = meshfile or ""
        colorid = colorid or (shiptype == "hive" and 0) or GetShipPurchaseColor()
        SetViewObject(modelview, meshname, meshfile, colorid, typeid, nil, true)
        if not meshname then return end
        if not typeid then
            iup.GetParent(ships_dropdown):hide()
            iup.GetParent(button_next):hide()
            return
        else
            iup.GetParent(ships_dropdown).visible = "YES"
            iup.GetParent(button_next).visible = "YES"
        end

        -- stats dropdown
        for i = 1, stats_rowcount do
            stats_dropdown[i] = nil
        end
        stats_dropdown[1] = "No stats available"
        stats_rowcount = 1
        if typeid ~= 0 then
            local shipItem = InvManager.GetInventoryItemPrototype(typeid)
            -- add stats from extended description
            local shipDesc = shipItem and shipItem:GetExtendedDesc() or ""
            local shipDescLines = {}
            for line in shipDesc:gmatch("(.-)\n") do
                --if line:match(":") then -- only need this for the dev furie
                table.insert(shipDescLines, line)
                --end
            end
            for i, line in ipairs(shipDescLines) do
                if line ~= "" then
                    stats_rowcount = i + 2
                    stats_dropdown[stats_rowcount] = line
                end
            end
            stats_dropdown[1] = "\tClick to see the stats"

            -- add more stats not in description
            local proto = InvManager.GetInventoryItemPrototype(typeid)
            local ship_parameters = {
                "RadarRange",
                "AwacsProx",
                "StormRadarRange",
                "SensorRange",
                "FogSensorRange",
                "StormSensorRange",
                "AddonScannerRange",
                "CargoScannerRange",
                "ScannerRange",
                "TempScannerRange",
            }
            local parameter_names = {
                RadarRange = "Radar Extender",
                StormRadarRange = "Storm Radar",
                AwacsProx = "Group Radar Extender",
                SensorRange = "Sensor Range",
                FogSensorRange = "Fog Sensor Range",
                StormSensorRange = "Storm Sensor Range",
                AddonScannerRange = "Addon Scanner",
                CargoScannerRange = "Cargo Scanner",
                ScannerRange = "Mineral Scanner",
                TempScannerRange = "Temperature Scanner",
            }
            for _, v in ipairs(ship_parameters) do
                local value = proto:GetParameter(v)
                if value then
                    stats_rowcount = stats_rowcount + 1
                    stats_dropdown[stats_rowcount] = parameter_names[v] ..
                        ": " .. value ..
                        ((v:match("Range") or v == "AwacsProx") and " m" or "")
                end
            end

            -- format all numbers
            for i = 1, stats_rowcount do
                if stats_dropdown[i] then
                    local formatted_line = stats_dropdown[i]:gsub("(%d+)", function(match)
                        return comma_value(tonumber(match))
                    end)
                    stats_dropdown[i] = formatted_line
                end
            end
        end

        -- livery dropdown
        for i = 1, liveries_rowcount do
            liveries_dropdown[i] = nil
        end
        liveries_dropdown.value = 0
        local liveries = {}
        liveries_row2overlayindex = {}
        -- you cannot skin ships that come with a default skin
        if not modelview.ReplacementMaterialSuffix then
            for skinid, skin_data in pairs(ShipSkinPrereqs) do
                if not skin_data.noship then
                    for shipmesh in pairs(skin_data) do
                        if shipmesh == meshname then
                            table.insert(liveries, skinid)
                        end
                    end
                end
            end
        end
        liveries_dropdown[1] = #liveries > 0 and "Select livery" or "No liveries"
        liveries_rowcount = 1
        for i, skinid in ipairs(liveries) do
            liveries_rowcount = i + 3
            liveries_dropdown[liveries_rowcount] = ShipSkinNames[skinid]
            liveries_row2overlayindex[liveries_rowcount] = skinid
        end
        if liveries_rowcount > 1 then liveries_dropdown[3] = "Basic" end
    end

    function dialog:SetLivery(overlayindex)
        overlayindex = overlayindex or 0
        -- hack to trigger a redraw of the base model if we don't overlay a skin
        if overlayindex == 0 then modelview.value = modelview.value .. "" end
        modelview.ReplacementMaterialSuffix = ShipSkins[overlayindex]
    end

    function dialog:Close()
        return Shipviewer.Close()
    end

    return dialog
end

function Shipviewer.Close()
    -- clear all the ship data and the dialog itself to free up memory
    Shipviewer.ships = nil
    Shipviewer.Dialog:hide()
    Shipviewer.Dialog = nil
    collectgarbage()
end

function Shipviewer.ResetStationTab()
    -- reset station tab if on the ship tab because two modelviews showing at
    -- the same time clash
    if StationEquipmentTab.visible == "YES" then
        StationTabs:SetTab(StationCommerceTab)
    end
end

function Shipviewer.ShowShips(small_screen)
    Shipviewer.ResetStationTab()
    if not Shipviewer.Dialog then
        if not pcall(dofile, "plugins/Shipviewer/shipdata.lua") then
            print("\127ff0000Ship data could not be loaded, please make " ..
                "sure this plugin is installed to plugins/Shipviewer and " ..
                "the file shipdata.lua is present.\127o")
            return
        end
        Shipviewer.Dialog = Shipviewer.CreateDialog(Shipviewer.ships.playable, small_screen)
    end
    Shipviewer.Dialog:show(iup.CENTER, iup.CENTER)
end

function Shipviewer.ShowModel(_, args)
    if args and args[1] then
        Shipviewer.ShowShips()
        Shipviewer.Dialog:SetShip(args[1], args[2] or args[1])
    end
end

RegisterUserCommand("shipviewer", Shipviewer.ShowShips)
RegisterUserCommand("shipviewer2", Shipviewer.ShowShips, true)
RegisterUserCommand("modelviewer", Shipviewer.ShowModel)
